@@ -13,7 +13,8 @@ Run tests if fftpack is not installed:
   python tests/test_pseudo_diffs.py [<level>]
 """
 
-from numpy.testing import *
+from numpy.testing import (TestCase, assert_equal, assert_almost_equal,
+                           assert_array_almost_equal, rand, run_module_suite)
 from scipy.fftpack import diff, fft, ifft, tilbert, itilbert, hilbert, \
                           ihilbert, shift, fftfreq, cs_diff, sc_diff, \
                           ss_diff, cc_diff
