@@ -3,7 +3,7 @@ from __future__ import division, print_function, absolute_import
 import numpy as np
 from numpy import cos, sin, pi
 from numpy.testing import TestCase, run_module_suite, assert_equal, \
-    assert_almost_equal, assert_allclose
+    assert_almost_equal, assert_allclose, assert_
 
 from scipy.integrate import quadrature, romberg, romb, newton_cotes, cumtrapz
 
@@ -27,6 +27,23 @@ class TestQuadrature(TestCase):
         table_val = 1e90 * 0.30614353532540296487
         assert_allclose(val, table_val, rtol=1e-10)
 
+    def test_quadrature_miniter(self):
+        # Typical function with two extra arguments:
+        def myfunc(x,n,z):       # Bessel function integrand
+            return cos(n*x-z*sin(x))/pi
+        table_val = 0.30614353532540296487
+        for miniter in [5, 52]:
+            val, err = quadrature(myfunc,0,pi,(2,1.8),miniter=miniter)
+            assert_almost_equal(val, table_val, decimal=7)
+            assert_(err < 1.0)
+
+    def test_quadrature_single_args(self):
+        def myfunc(x,n):
+            return 1e90 * cos(n*x-1.8*sin(x))/pi
+        val, err = quadrature(myfunc,0,pi,args=2,rtol=1e-10)
+        table_val = 1e90 * 0.30614353532540296487
+        assert_allclose(val, table_val, rtol=1e-10)
+
     def test_romberg(self):
         # Typical function with two extra arguments:
         def myfunc(x, n, z):       # Bessel function integrand
@@ -104,7 +121,7 @@ class TestCumtrapz(TestCase):
         y_int = cumtrapz(y, x, initial=None)
         assert_allclose(y_int, y_expected[1:])
 
-    def test_nd(self):
+    def test_y_nd_x_nd(self):
         x = np.arange(3 * 2 * 4).reshape(3, 2, 4)
         y = x
         y_int = cumtrapz(y, x, initial=0)
@@ -125,6 +142,29 @@ class TestCumtrapz(TestCase):
             y_int = cumtrapz(y, x, initial=None, axis=axis)
             assert_equal(y_int.shape, shape)
 
+    def test_y_nd_x_1d(self):
+        y = np.arange(3 * 2 * 4).reshape(3, 2, 4)
+        x = np.arange(4)**2
+        # Try with all axes
+        ys_expected = (
+            np.array([[[ 4.,  5.,  6.,  7.],
+                       [ 8.,  9., 10., 11.]],
+                      [[40., 44., 48., 52.],
+                       [56., 60., 64., 68.]]]),
+            np.array([[[ 2.,  3.,  4.,  5.]],
+                      [[10., 11., 12., 13.]],
+                      [[18., 19., 20., 21.]]]),
+            np.array([[[ 0.5,  5.,  17.5],
+                       [ 4.5, 21.,  53.5]],
+                      [[ 8.5, 37.,  89.5],
+                       [12.5, 53., 125.5]],
+                      [[16.5, 69., 161.5],
+                       [20.5, 85., 197.5]]]))
+
+        for axis, y_expected in zip([0, 1, 2], ys_expected):
+            y_int = cumtrapz(y, x=x[:y.shape[axis]], axis=axis, initial=None)
+            assert_allclose(y_int, y_expected)
+
     def test_x_none(self):
         y = np.linspace(-2, 2, num=5)
 
