@@ -10,7 +10,7 @@ from __future__ import division, print_function, absolute_import
 import sys
 import operator
 
-from scipy.lib.six.moves import reduce
+from scipy.lib.six import reduce
 
 import numpy as np
 
@@ -35,6 +35,7 @@ class MatWriteError(Exception):
 class MatReadWarning(UserWarning):
     pass
 
+
 doc_dict = \
     {'file_arg':
          '''file_name : str
@@ -300,7 +301,7 @@ def matdims(arr, oned_as='column'):
     shape = arr.shape
     if shape == ():  # scalar
         return (1,1)
-    if reduce(operator.mul, shape) == 0: # zero elememts
+    if reduce(operator.mul, shape) == 0:  # zero elememts
         return (0,) * np.max([arr.ndim, 2])
     if len(shape) == 1:  # 1D
         if oned_as == 'column':
@@ -344,7 +345,8 @@ class MatFileReader(object):
                  squeeze_me=False,
                  chars_as_strings=True,
                  matlab_compatible=False,
-                 struct_as_record=True
+                 struct_as_record=True,
+                 verify_compressed_data_integrity=True
                  ):
         '''
         Initializer for mat file reader
@@ -368,6 +370,7 @@ class MatFileReader(object):
             self.squeeze_me = squeeze_me
             self.chars_as_strings = chars_as_strings
             self.mat_dtype = mat_dtype
+        self.verify_compressed_data_integrity = verify_compressed_data_integrity
 
     def set_matlab_compatible(self):
         ''' Sets options to return arrays as MATLAB loads them '''
