@@ -1,6 +1,9 @@
 #!/usr/bin/env python
+from __future__ import division, print_function, absolute_import
 
-from tempfile import mktemp
+from tempfile import mkdtemp, mktemp
+import os
+import shutil
 from numpy import array,transpose
 from numpy.testing import TestCase, run_module_suite, assert_array_almost_equal, \
             assert_equal, rand
@@ -8,12 +11,17 @@ from numpy.testing import TestCase, run_module_suite, assert_array_almost_equal,
 import scipy.sparse
 from scipy.io.mmio import mminfo,mmread,mmwrite
 
-
 class TestMMIOArray(TestCase):
+    def setUp(self):
+        self.tmpdir = mkdtemp()
+        self.fn = os.path.join(self.tmpdir, 'testfile.mtx')
+
+    def tearDown(self):
+        shutil.rmtree(self.tmpdir)
 
     def test_simple(self):
         a = [[1,2],[3,4]]
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(2,2,4,'array','integer','general'))
         b = mmread(fn)
@@ -21,7 +29,7 @@ class TestMMIOArray(TestCase):
 
     def test_simple_rectangular(self):
         a = [[1,2,3],[4,5,6]]
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(2,3,6,'array','integer','general'))
         b = mmread(fn)
@@ -29,7 +37,7 @@ class TestMMIOArray(TestCase):
 
     def test_simple_rectangular_real(self):
         a = [[1,2],[3.5,4],[5,6]]
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(3,2,6,'array','real','general'))
         b = mmread(fn)
@@ -37,7 +45,7 @@ class TestMMIOArray(TestCase):
 
     def test_simple_real(self):
         a = [[1,2],[3,4.0]]
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(2,2,4,'array','real','general'))
         b = mmread(fn)
@@ -45,7 +53,7 @@ class TestMMIOArray(TestCase):
 
     def test_simple_complex(self):
         a = [[1,2],[3,4j]]
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(2,2,4,'array','complex','general'))
         b = mmread(fn)
@@ -53,7 +61,7 @@ class TestMMIOArray(TestCase):
 
     def test_simple_symmetric(self):
         a = [[1,2],[2,4]]
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(2,2,4,'array','integer','symmetric'))
         b = mmread(fn)
@@ -61,7 +69,7 @@ class TestMMIOArray(TestCase):
 
     def test_simple_skew_symmetric(self):
         a = [[1,2],[-2,4]]
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(2,2,4,'array','integer','skew-symmetric'))
         b = mmread(fn)
@@ -69,7 +77,7 @@ class TestMMIOArray(TestCase):
 
     def test_simple_skew_symmetric_float(self):
         a = array([[1,2],[-2.0,4]],'f')
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(2,2,4,'array','real','skew-symmetric'))
         b = mmread(fn)
@@ -77,7 +85,7 @@ class TestMMIOArray(TestCase):
 
     def test_simple_hermitian(self):
         a = [[1,2+3j],[2-3j,4]]
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(2,2,4,'array','complex','hermitian'))
         b = mmread(fn)
@@ -87,7 +95,7 @@ class TestMMIOArray(TestCase):
         sz = (20,20)
         a = rand(*sz)
         a = a + transpose(a)
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(20,20,400,'array','real','symmetric'))
         b = mmread(fn)
@@ -96,7 +104,7 @@ class TestMMIOArray(TestCase):
     def test_random_rect_real(self):
         sz = (20,15)
         a = rand(*sz)
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,a)
         assert_equal(mminfo(fn),(20,15,300,'array','real','general'))
         b = mmread(fn)
@@ -184,79 +192,82 @@ _symmetric_pattern_example = '''\
     5     4
 '''
 
+
 class TestMMIOCoordinate(TestCase):
+    def setUp(self):
+        self.tmpdir = mkdtemp()
+        self.fn = os.path.join(self.tmpdir, 'testfile.mtx')
+
+    def tearDown(self):
+        shutil.rmtree(self.tmpdir)
+
     def test_read_general(self):
-        """read a general matrix"""
-        fn = mktemp()
+        fn = self.fn
         f = open(fn,'w')
         f.write(_general_example)
         f.close()
         assert_equal(mminfo(fn),(5,5,8,'coordinate','real','general'))
-        a = [[1,    0,      0,       6,      0],
-             [0,   10.5,    0,       0,      0],
-             [0,    0,    .015,      0,      0],
-             [0,  250.5,    0,     -280,    33.32],
-             [0,    0,      0,       0,     12]]
+        a = [[1, 0, 0, 6, 0],
+             [0, 10.5, 0, 0, 0],
+             [0, 0, .015, 0, 0],
+             [0, 250.5, 0, -280, 33.32],
+             [0, 0, 0, 0, 12]]
         b = mmread(fn).todense()
         assert_array_almost_equal(a,b)
 
     def test_read_hermitian(self):
-        """read a hermitian matrix"""
-        fn = mktemp()
+        fn = self.fn
         f = open(fn,'w')
         f.write(_hermitian_example)
         f.close()
         assert_equal(mminfo(fn),(5,5,7,'coordinate','complex','hermitian'))
-        a = [[1,      0,               0,       0,              0],
-             [0,     10.5,             0,    250.5 - 22.22j,    0],
-             [0,      0,            .015,       0,              0],
-             [0,  250.5 + 22.22j,      0,      -280,          -33.32j],
-             [0,      0,               0,     33.32j,          12]]
+        a = [[1, 0, 0, 0, 0],
+             [0, 10.5, 0, 250.5 - 22.22j, 0],
+             [0, 0, .015, 0, 0],
+             [0, 250.5 + 22.22j, 0, -280, -33.32j],
+             [0, 0, 0, 33.32j, 12]]
         b = mmread(fn).todense()
         assert_array_almost_equal(a,b)
 
     def test_read_skew(self):
-        """read a skew-symmetric matrix"""
-        fn = mktemp()
+        fn = self.fn
         f = open(fn,'w')
         f.write(_skew_example)
         f.close()
         assert_equal(mminfo(fn),(5,5,7,'coordinate','real','skew-symmetric'))
-        a = [[1,      0,               0,       0,     0],
-             [0,     10.5,             0,  -250.5,     0],
-             [0,      0,            .015,       0,     0],
-             [0,  250.5,               0,    -280,     0],
-             [0,      0,               0,       0,    12]]
+        a = [[1, 0, 0, 0, 0],
+             [0, 10.5, 0, -250.5, 0],
+             [0, 0, .015, 0, 0],
+             [0, 250.5, 0, -280, 0],
+             [0, 0, 0, 0, 12]]
         b = mmread(fn).todense()
         assert_array_almost_equal(a,b)
 
     def test_read_symmetric(self):
-        """read a symmetric matrix"""
-        fn = mktemp()
+        fn = self.fn
         f = open(fn,'w')
         f.write(_symmetric_example)
         f.close()
         assert_equal(mminfo(fn),(5,5,7,'coordinate','real','symmetric'))
-        a = [[1,      0,               0,       0,     0],
-             [0,     10.5,             0,   250.5,     0],
-             [0,      0,            .015,       0,     0],
-             [0,  250.5,               0,    -280,     8],
-             [0,      0,               0,       8,    12]]
+        a = [[1, 0, 0, 0, 0],
+             [0, 10.5, 0, 250.5, 0],
+             [0, 0, .015, 0, 0],
+             [0, 250.5, 0, -280, 8],
+             [0, 0, 0, 8, 12]]
         b = mmread(fn).todense()
         assert_array_almost_equal(a,b)
 
     def test_read_symmetric_pattern(self):
-        """read a symmetric pattern matrix"""
-        fn = mktemp()
+        fn = self.fn
         f = open(fn,'w')
         f.write(_symmetric_pattern_example)
         f.close()
         assert_equal(mminfo(fn),(5,5,7,'coordinate','pattern','symmetric'))
-        a = [[1,     0,     0,     0,     0],
-             [0,     1,     0,     1,     0],
-             [0,     0,     1,     0,     0],
-             [0,     1,     0,     1,     1],
-             [0,     0,     0,     1,     1]]
+        a = [[1, 0, 0, 0, 0],
+             [0, 1, 0, 1, 0],
+             [0, 0, 1, 0, 0],
+             [0, 1, 0, 1, 1],
+             [0, 0, 0, 1, 1]]
         b = mmread(fn).todense()
         assert_array_almost_equal(a,b)
 
@@ -264,7 +275,7 @@ class TestMMIOCoordinate(TestCase):
         #http://projects.scipy.org/scipy/ticket/883
 
         b = scipy.sparse.coo_matrix((10,10))
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,b)
 
         assert_equal(mminfo(fn),(10,10,0,'coordinate','real','general'))
@@ -272,15 +283,64 @@ class TestMMIOCoordinate(TestCase):
         b = mmread(fn).todense()
         assert_array_almost_equal(a,b)
 
+    def test_bzip2_py3(self):
+        # test if fix for #2152 works
+        try:
+            # bz2 module isn't always built when building Python.
+            import bz2
+        except:
+            return
+        I = array([0, 0, 1, 2, 3, 3, 3, 4])
+        J = array([0, 3, 1, 2, 1, 3, 4, 4])
+        V = array([1.0, 6.0, 10.5, 0.015, 250.5, -280.0, 33.32, 12.0])
+
+        b = scipy.sparse.coo_matrix((V,(I,J)),shape=(5,5))
+
+        fn = self.fn
+        mmwrite(fn, b)
+
+        fn_bzip2 = "%s.bz2" % fn
+        with open(fn, 'rb') as f_in:
+            f_out = bz2.BZ2File(fn_bzip2, 'wb')
+            f_out.write(f_in.read())
+            f_out.close()
+
+        a = mmread(fn_bzip2).todense()
+        assert_array_almost_equal(a, b.todense())
+
+    def test_gzip_py3(self):
+        # test if fix for #2152 works
+        try:
+            # gzip module can be missing from Python installation
+            import gzip
+        except:
+            return
+        I = array([0, 0, 1, 2, 3, 3, 3, 4])
+        J = array([0, 3, 1, 2, 1, 3, 4, 4])
+        V = array([1.0, 6.0, 10.5, 0.015, 250.5, -280.0, 33.32, 12.0])
+
+        b = scipy.sparse.coo_matrix((V,(I,J)),shape=(5,5))
+
+        fn = self.fn
+        mmwrite(fn, b)
+
+        fn_gzip = "%s.gz" % fn
+        with open(fn, 'rb') as f_in:
+            f_out = gzip.open(fn_gzip, 'wb')
+            f_out.write(f_in.read())
+            f_out.close()
+
+        a = mmread(fn_gzip).todense()
+        assert_array_almost_equal(a, b.todense())
 
     def test_real_write_read(self):
         I = array([0, 0, 1, 2, 3, 3, 3, 4])
         J = array([0, 3, 1, 2, 1, 3, 4, 4])
-        V = array([  1.0,   6.0,   10.5, 0.015,   250.5,  -280.0, 33.32, 12.0 ])
+        V = array([1.0, 6.0, 10.5, 0.015, 250.5, -280.0, 33.32, 12.0])
 
         b = scipy.sparse.coo_matrix((V,(I,J)),shape=(5,5))
 
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,b)
 
         assert_equal(mminfo(fn),(5,5,8,'coordinate','real','general'))
@@ -291,12 +351,12 @@ class TestMMIOCoordinate(TestCase):
     def test_complex_write_read(self):
         I = array([0, 0, 1, 2, 3, 3, 3, 4])
         J = array([0, 3, 1, 2, 1, 3, 4, 4])
-        V = array([  1.0 + 3j,    6.0 + 2j,  10.50 + 0.9j, 0.015 + -4.4j,
-                   250.5 + 0j, -280.0 + 5j,  33.32 + 6.4j, 12.00 + 0.8j])
+        V = array([1.0 + 3j, 6.0 + 2j, 10.50 + 0.9j, 0.015 + -4.4j,
+                   250.5 + 0j, -280.0 + 5j, 33.32 + 6.4j, 12.00 + 0.8j])
 
         b = scipy.sparse.coo_matrix((V,(I,J)),shape=(5,5))
 
-        fn = mktemp()
+        fn = self.fn
         mmwrite(fn,b)
 
         assert_equal(mminfo(fn),(5,5,8,'coordinate','complex','general'))
@@ -310,17 +370,17 @@ class TestMMIOCoordinate(TestCase):
         I = array([0, 0, 1, 2, 3, 3, 3, 4])
         J = array([0, 3, 1, 2, 1, 3, 4, 4])
 
-        V = array([  1.0,   6.0,   10.5, 0.015,   250.5,  -280.0, 33.32, 12.0 ])
-        mats.append( scipy.sparse.coo_matrix((V,(I,J)),shape=(5,5)) )
+        V = array([1.0, 6.0, 10.5, 0.015, 250.5, -280.0, 33.32, 12.0])
+        mats.append(scipy.sparse.coo_matrix((V,(I,J)),shape=(5,5)))
 
-        V = array([  1.0 + 3j,    6.0 + 2j,  10.50 + 0.9j, 0.015 + -4.4j,
-                   250.5 + 0j, -280.0 + 5j,  33.32 + 6.4j, 12.00 + 0.8j])
-        mats.append( scipy.sparse.coo_matrix((V,(I,J)),shape=(5,5)) )
+        V = array([1.0 + 3j, 6.0 + 2j, 10.50 + 0.9j, 0.015 + -4.4j,
+                   250.5 + 0j, -280.0 + 5j, 33.32 + 6.4j, 12.00 + 0.8j])
+        mats.append(scipy.sparse.coo_matrix((V,(I,J)),shape=(5,5)))
 
         for mat in mats:
             expected = mat.todense()
             for fmt in ['csr','csc','coo']:
-                fn = mktemp()
+                fn = mktemp(dir=self.tmpdir) # safe, we own tmpdir
                 mmwrite(fn, mat.asformat(fmt))
 
                 result = mmread(fn).todense()
