@@ -92,15 +92,15 @@ _extra_keywords_doc = \
     dict of extra keyword arguments to pass to passed function"""
 
 docdict = {
-    'input':_input_doc,
-    'axis':_axis_doc,
-    'output':_output_doc,
-    'size_foot':_size_foot_doc,
-    'mode':_mode_doc,
-    'cval':_cval_doc,
-    'origin':_origin_doc,
-    'extra_arguments':_extra_arguments_doc,
-    'extra_keywords':_extra_keywords_doc,
+    'input': _input_doc,
+    'axis': _axis_doc,
+    'output': _output_doc,
+    'size_foot': _size_foot_doc,
+    'mode': _mode_doc,
+    'cval': _cval_doc,
+    'origin': _origin_doc,
+    'extra_arguments': _extra_arguments_doc,
+    'extra_keywords': _extra_keywords_doc,
     }
 
 docfiller = doccer.filldoc(docdict)
@@ -178,7 +178,7 @@ def convolve1d(input, weights, axis=-1, output=None, mode="reflect",
 
 @docfiller
 def gaussian_filter1d(input, sigma, axis=-1, order=0, output=None,
-                      mode="reflect", cval=0.0):
+                      mode="reflect", cval=0.0, truncate=4.0):
     """One-dimensional Gaussian filter.
 
     Parameters
@@ -195,6 +195,9 @@ def gaussian_filter1d(input, sigma, axis=-1, order=0, output=None,
     %(output)s
     %(mode)s
     %(cval)s
+    truncate : float
+        Truncate the filter at this many standard deviations.
+        Default is 4.0.
 
     Returns
     -------
@@ -204,9 +207,8 @@ def gaussian_filter1d(input, sigma, axis=-1, order=0, output=None,
     if order not in range(4):
         raise ValueError('Order outside 0..3 not implemented')
     sd = float(sigma)
-    # make the length of the filter equal to 4 times the standard
-    # deviations:
-    lw = int(4.0 * sd + 0.5)
+    # make the radius of the filter equal to truncate standard deviations
+    lw = int(truncate * sd + 0.5)
     weights = [0.0] * (2 * lw + 1)
     weights[lw] = 1.0
     sum = 1.0
@@ -247,7 +249,7 @@ def gaussian_filter1d(input, sigma, axis=-1, order=0, output=None,
 
 @docfiller
 def gaussian_filter(input, sigma, order=0, output=None,
-                  mode="reflect", cval=0.0):
+                  mode="reflect", cval=0.0, truncate=4.0):
     """Multidimensional Gaussian filter.
 
     Parameters
@@ -268,6 +270,9 @@ def gaussian_filter(input, sigma, order=0, output=None,
     %(output)s
     %(mode)s
     %(cval)s
+    truncate : float
+        Truncate the filter at this many standard deviations.
+        Default is 4.0.
 
     Returns
     -------
@@ -296,7 +301,7 @@ def gaussian_filter(input, sigma, order=0, output=None,
     if len(axes) > 0:
         for axis, sigma, order in axes:
             gaussian_filter1d(input, sigma, axis, order, output,
-                              mode, cval)
+                              mode, cval, truncate)
             input = output
     else:
         output[...] = input[...]
@@ -405,7 +410,7 @@ def laplace(input, output=None, mode="reflect", cval=0.0):
 
 @docfiller
 def gaussian_laplace(input, sigma, output=None, mode="reflect",
-                     cval=0.0):
+                     cval=0.0, **kwargs):
     """Multidimensional Laplace filter using gaussian second derivatives.
 
     Parameters
@@ -418,15 +423,19 @@ def gaussian_laplace(input, sigma, output=None, mode="reflect",
     %(output)s
     %(mode)s
     %(cval)s
+    Extra keyword arguments will be passed to gaussian_filter().
     """
     input = numpy.asarray(input)
 
-    def derivative2(input, axis, output, mode, cval, sigma):
+    def derivative2(input, axis, output, mode, cval, sigma, **kwargs):
         order = [0] * input.ndim
         order[axis] = 2
-        return gaussian_filter(input, sigma, order, output, mode, cval)
+        return gaussian_filter(input, sigma, order, output, mode, cval,
+                               **kwargs)
+
     return generic_laplace(input, derivative2, output, mode, cval,
-                           extra_arguments=(sigma,))
+                           extra_arguments=(sigma,),
+                           extra_keywords=kwargs)
 
 
 @docfiller
@@ -480,7 +489,7 @@ def generic_gradient_magnitude(input, derivative, output=None,
 
 @docfiller
 def gaussian_gradient_magnitude(input, sigma, output=None,
-                mode="reflect", cval=0.0):
+                mode="reflect", cval=0.0, **kwargs):
     """Multidimensional gradient magnitude using Gaussian derivatives.
 
     Parameters
@@ -493,15 +502,19 @@ def gaussian_gradient_magnitude(input, sigma, output=None,
     %(output)s
     %(mode)s
     %(cval)s
+    Extra keyword arguments will be passed to gaussian_filter().
     """
     input = numpy.asarray(input)
 
-    def derivative(input, axis, output, mode, cval, sigma):
+    def derivative(input, axis, output, mode, cval, sigma, **kwargs):
         order = [0] * input.ndim
         order[axis] = 1
-        return gaussian_filter(input, sigma, order, output, mode, cval)
+        return gaussian_filter(input, sigma, order, output, mode,
+                               cval, **kwargs)
+
     return generic_gradient_magnitude(input, derivative, output, mode,
-                            cval, extra_arguments=(sigma,))
+                                      cval, extra_arguments=(sigma,),
+                                      extra_keywords=kwargs)
 
 
 def _correlate_or_convolve(input, weights, output, mode, cval, origin,
@@ -810,6 +823,13 @@ def maximum_filter1d(input, size, axis=-1, output=None,
     %(mode)s
     %(cval)s
     %(origin)s
+
+    Returns
+    -------
+    maximum1d : ndarray, None
+        Maximum-filtered array with same shape as input.
+        None if `output` is not None
+
     """
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
@@ -836,7 +856,7 @@ def _min_or_max_filter(input, size, footprint, structure, output, mode,
         else:
             footprint = numpy.asarray(footprint)
             footprint = footprint.astype(bool)
-            if numpy.alltrue(numpy.ravel(footprint),axis=0):
+            if numpy.alltrue(numpy.ravel(footprint), axis=0):
                 size = footprint.shape
                 footprint = None
                 separable = True
