@@ -1,6 +1,6 @@
 """
 This module implements the Sequential Least SQuares Programming optimization
-algorithm (SLSQP), orginally developed by Dieter Kraft.
+algorithm (SLSQP), originally developed by Dieter Kraft.
 See http://www.netlib.org/toms/733
 
 Functions
@@ -20,7 +20,7 @@ __all__ = ['approx_jacobian','fmin_slsqp']
 from scipy.optimize._slsqp import slsqp
 from numpy import zeros, array, linalg, append, asfarray, concatenate, finfo, \
                   sqrt, vstack, exp, inf, where, isfinite, atleast_1d
-from .optimize import wrap_function, Result, _check_unknown_options
+from .optimize import wrap_function, OptimizeResult, _check_unknown_options
 
 __docformat__ = "restructuredtext en"
 
@@ -61,13 +61,15 @@ def approx_jacobian(x,func,epsilon,*args):
         dx[i] = epsilon
         jac[i] = (func(*((x0+dx,)+args)) - f0)/epsilon
         dx[i] = 0.0
+
     return jac.transpose()
 
 
-def fmin_slsqp(func, x0, eqcons=[], f_eqcons=None, ieqcons=[], f_ieqcons=None,
-                bounds=[], fprime=None, fprime_eqcons=None,
-                fprime_ieqcons=None, args=(), iter = 100, acc = 1.0E-6,
-                iprint = 1, disp = None, full_output = 0, epsilon = _epsilon):
+def fmin_slsqp(func, x0, eqcons=(), f_eqcons=None, ieqcons=(), f_ieqcons=None,
+               bounds=(), fprime=None, fprime_eqcons=None,
+               fprime_ieqcons=None, args=(), iter=100, acc=1.0E-6,
+               iprint=1, disp=None, full_output=0, epsilon=_epsilon,
+               callback=None):
     """
     Minimize a function using Sequential Least SQuares Programming
 
@@ -132,6 +134,9 @@ def fmin_slsqp(func, x0, eqcons=[], f_eqcons=None, ieqcons=[], f_ieqcons=None,
         information.
     epsilon : float
         The step size for finite-difference derivative estimates.
+    callback : callable, optional
+        Called after each iteration, as ``callback(x)``, where ``x`` is the
+        current parameter vector.
 
     Returns
     -------
@@ -178,7 +183,8 @@ def fmin_slsqp(func, x0, eqcons=[], f_eqcons=None, ieqcons=[], f_ieqcons=None,
             'ftol': acc,
             'iprint': iprint,
             'disp': iprint != 0,
-            'eps': epsilon}
+            'eps': epsilon,
+            'callback': callback}
 
     # Build the constraints as a tuple of dictionaries
     cons = ()
@@ -207,7 +213,7 @@ def fmin_slsqp(func, x0, eqcons=[], f_eqcons=None, ieqcons=[], f_ieqcons=None,
 def _minimize_slsqp(func, x0, args=(), jac=None, bounds=None,
                     constraints=(),
                     maxiter=100, ftol=1.0E-6, iprint=1, disp=False,
-                    eps=_epsilon,
+                    eps=_epsilon, callback=None,
                     **unknown_options):
     """
     Minimize a scalar function of one or more variables using Sequential
@@ -396,6 +402,10 @@ def _minimize_slsqp(func, x0, args=(), jac=None, bounds=None,
         # Call SLSQP
         slsqp(m, meq, x, xl, xu, fx, c, g, a, acc, majiter, mode, w, jw)
 
+        # call callback if major iteration has incremented
+        if callback is not None and majiter > majiter_prev:
+            callback(x)
+
         # Print the status of the current iterate if iprint > 2 and the
         # major iteration has incremented
         if iprint >= 2 and majiter > majiter_prev:
@@ -416,9 +426,10 @@ def _minimize_slsqp(func, x0, args=(), jac=None, bounds=None,
         print("            Function evaluations:", feval[0])
         print("            Gradient evaluations:", geval[0])
 
-    return Result(x=x, fun=fx, jac=g, nit=int(majiter), nfev=feval[0],
-                  njev=geval[0], status=int(mode),
-                  message=exit_modes[int(mode)], success=(mode == 0))
+    return OptimizeResult(x=x, fun=fx, jac=g, nit=int(majiter), nfev=feval[0],
+                          njev=geval[0], status=int(mode),
+                          message=exit_modes[int(mode)], success=(mode == 0))
+
 
 if __name__ == '__main__':
 
