@@ -772,8 +772,8 @@ static void fill_buffer(char *ip1, PyArrayObject *ap1, PyArrayObject *ap2, char
   int ndims = ap1->nd;
   intp *dims2 = ap2->dimensions;
   intp *dims1 = ap1->dimensions;
-  intp is1 = ap1->strides[ndims-1];
-  intp is2 = ap2->strides[ndims-1];
+  intp is1 = ap1->descr->elsize;
+  intp is2 = ap2->descr->elsize;
   char *ip2 = ap2->data;
   int elsize = ap1->descr->elsize;
   char *ptr;
@@ -1009,7 +1009,7 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
     int mode=2, boundary=0, typenum, flag, flip=1, ret;
     intp *aout_dimens=NULL, *dims=NULL;
     char zeros[32];  /* Zeros */
-    int n1, n2, i;
+    int i;
     PyArrayObject *ain1=NULL, *ain2=NULL, *aout=NULL;
     PyArrayObject *afill=NULL, *newfill=NULL;
 
@@ -1031,7 +1031,7 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
 	    newfill = (PyArrayObject *)PyArray_SimpleNewFromData(0, dims, typenum, zeros);
 	}
 	else {
-	    afill = (PyArrayObject *)PyArray_FromObject(fill_value, PyArray_CDOUBLE, 0, 0);
+	    afill = (PyArrayObject *)PyArray_FromObject(fill_value, NPY_CDOUBLE, 0, 0);
 	    if (afill == NULL) goto fail;
 	    newfill = (PyArrayObject *)PyArray_Cast(afill, typenum);
 	}
@@ -1041,10 +1041,7 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
 	newfill = (PyArrayObject *)PyArray_SimpleNewFromData(0, dims, typenum, zeros);
 	if (newfill == NULL) goto fail;
     }
-    
-    n1 = PyArray_Size((PyObject *)ain1);
-    n2 = PyArray_Size((PyObject *)ain2);
-    
+
     aout_dimens = malloc(ain1->nd*sizeof(intp));
     switch(mode & OUTSIZE_MASK) {
     case VALID:
@@ -1088,6 +1085,7 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
 
     switch (ret) {
     case 0:
+      free(aout_dimens);
       Py_DECREF(ain1);
       Py_DECREF(ain2);
       Py_XDECREF(afill);
@@ -1166,11 +1164,11 @@ static PyObject *sigtools_remez(PyObject *NPY_UNUSED(dummy), PyObject *args) {
 	}
 	
 	
-	a_bands = (PyArrayObject *)PyArray_ContiguousFromObject(bands, PyArray_DOUBLE,1,1);
+	a_bands = (PyArrayObject *)PyArray_ContiguousFromObject(bands, NPY_DOUBLE,1,1);
 	if (a_bands == NULL) goto fail;
-	a_des = (PyArrayObject *)PyArray_ContiguousFromObject(des, PyArray_DOUBLE,1,1);
+	a_des = (PyArrayObject *)PyArray_ContiguousFromObject(des, NPY_DOUBLE,1,1);
 	if (a_des == NULL) goto fail;
-	a_weight = (PyArrayObject *)PyArray_ContiguousFromObject(weight, PyArray_DOUBLE,1,1);
+	a_weight = (PyArrayObject *)PyArray_ContiguousFromObject(weight, NPY_DOUBLE,1,1);
 	if (a_weight == NULL) goto fail;
 
 
@@ -1203,7 +1201,7 @@ static PyObject *sigtools_remez(PyObject *NPY_UNUSED(dummy), PyObject *args) {
 	}
 
 	ret_dimens = numtaps;
-	h = (PyArrayObject *)PyArray_SimpleNew(1, &ret_dimens, PyArray_DOUBLE);
+	h = (PyArrayObject *)PyArray_SimpleNew(1, &ret_dimens, NPY_DOUBLE);
 	if (h == NULL) goto fail;
 
 	err=pre_remez((double *)h->data, numtaps, numbands, (double *)a_bands->data, (double *)a_des->data, (double *)a_weight->data, type, maxiter, grid_density);
@@ -1270,13 +1268,13 @@ static PyObject *sigtools_median2d(PyObject *NPY_UNUSED(dummy), PyObject *args)
     }
     else {
 	switch (typenum) {
-	case PyArray_UBYTE:
+	case NPY_UBYTE:
 	    b_medfilt2((unsigned char *)DATA(a_image), (unsigned char *)DATA(a_out), Nwin, DIMS(a_image));
 	    break;
-	case PyArray_FLOAT:
+	case NPY_FLOAT:
 	    f_medfilt2((float *)DATA(a_image), (float *)DATA(a_out), Nwin, DIMS(a_image));
 	    break;
-	case PyArray_DOUBLE:
+	case NPY_DOUBLE:
 	    d_medfilt2((double *)DATA(a_image), (double *)DATA(a_out), Nwin, DIMS(a_image));
 	    break;
 	default:
@@ -1326,7 +1324,7 @@ static struct PyModuleDef moduledef = {
 };
 PyObject *PyInit_sigtools(void)
 {
-    PyObject *m, *d, *s;
+    PyObject *m;
 
     m = PyModule_Create(&moduledef);
 	import_array();
@@ -1339,10 +1337,8 @@ PyObject *PyInit_sigtools(void)
 /* Initialization function for the module (*must* be called initsigtools) */
 
 PyMODINIT_FUNC initsigtools(void) {
-        PyObject *m, *d;
-	
-	/* Create the module and add the functions */
-	m = Py_InitModule("sigtools", toolbox_module_methods);
+    /* Create the module and add the functions */
+    Py_InitModule("sigtools", toolbox_module_methods);
 
 	/* Import the C API function pointers for the Array Object*/
 	import_array();
@@ -1354,15 +1350,7 @@ PyMODINIT_FUNC initsigtools(void) {
 	PyImport_ImportModule("numpy.core.multiarray");
 	/* { PyObject *multi = PyImport_ImportModule("multiarray"); } */
 
-	/* Add some symbolic constants to the module */
-	d = PyModule_GetDict(m);
-
-	/* PyDict_SetItemString(d,"BANDPASS", PyInt_FromLong((long) BANDPASS));
-        PyDict_SetItemString(d,"DIFFERENTIATOR", PyInt_FromLong((long) DIFFERENTIATOR));
-        PyDict_SetItemString(d,"HILBERT", PyInt_FromLong((long) HILBERT));
-        */
-
-        scipy_signal_sigtools_linear_filter_module_init();
+    scipy_signal_sigtools_linear_filter_module_init();
 
 	/* Check for errors */
 	if (PyErr_Occurred()) {
