@@ -1,28 +1,34 @@
+from __future__ import division, print_function, absolute_import
+
 from warnings import warn
 
-from numpy import asarray
-from scipy.sparse import isspmatrix_csc, isspmatrix_csr, isspmatrix, \
-        SparseEfficiencyWarning, csc_matrix
+import numpy as np
+from numpy import asarray, empty, ravel, nonzero
+from scipy.sparse import (isspmatrix_csc, isspmatrix_csr, isspmatrix,
+                          SparseEfficiencyWarning, csc_matrix)
 
-import _superlu
+from . import _superlu
 
 noScikit = False
 try:
     import scikits.umfpack as umfpack
 except ImportError:
-    import umfpack
     noScikit = True
 
-isUmfpack = hasattr( umfpack, 'UMFPACK_OK' )
+useUmfpack = not noScikit
 
-useUmfpack = True
+__all__ = ['use_solver', 'spsolve', 'splu', 'spilu', 'factorized',
+           'MatrixRankWarning']
 
 
-__all__ = [ 'use_solver', 'spsolve', 'splu', 'spilu', 'factorized' ]
+class MatrixRankWarning(UserWarning):
+    pass
 
-def use_solver( **kwargs ):
+
+def use_solver(**kwargs):
     """
-    Valid keyword arguments with defaults (other ignored):
+    Valid keyword arguments with defaults (other ignored)::
+
       useUmfpack = True
       assumeSortedIndices = False
 
@@ -31,72 +37,136 @@ def use_solver( **kwargs ):
     based solver to be used.
 
     Umfpack requires a CSR/CSC matrix to have sorted column/row indices. If
-    sure that the matrix fulfills this, pass assumeSortedIndices=True
+    sure that the matrix fulfills this, pass ``assumeSortedIndices=True``
     to gain some speed.
+
     """
     if 'useUmfpack' in kwargs:
         globals()['useUmfpack'] = kwargs['useUmfpack']
 
-    if isUmfpack:
-        umfpack.configure( **kwargs )
+    #TODO: pass other options to scikit
 
 
 def spsolve(A, b, permc_spec=None, use_umfpack=True):
-    """Solve the sparse linear system Ax=b
+    """Solve the sparse linear system Ax=b, where b may be a vector or a matrix.
+
+    Parameters
+    ----------
+    A : ndarray or sparse matrix
+        The square matrix A will be converted into CSC or CSR form
+    b : ndarray or sparse matrix
+        The matrix or vector representing the right hand side of the equation.
+        If a vector, b.size must
+    permc_spec : str, optional
+        How to permute the columns of the matrix for sparsity preservation.
+        (default: 'COLAMD')
+
+        - ``NATURAL``: natural ordering.
+        - ``MMD_ATA``: minimum degree ordering on the structure of A^T A.
+        - ``MMD_AT_PLUS_A``: minimum degree ordering on the structure of A^T+A.
+        - ``COLAMD``: approximate minimum degree column ordering
+    use_umfpack : bool (optional)
+        if True (default) then use umfpack for the solution.  This is
+        only referenced if b is a vector.
+
+    Returns
+    -------
+    x : ndarray or sparse matrix
+        the solution of the sparse linear equation.
+        If b is a vector, then x is a vector of size A.shape[1]
+        If b is a matrix, then x is a matrix of size (A.shape[1], b.shape[1])
+
+    Notes
+    -----
+    For solving the matrix expression AX = B, this solver assumes the resulting
+    matrix X is sparse, as is often the case for very sparse inputs.  If the
+    resulting X is dense, the construction of this sparse result will be
+    relatively expensive.  In that case, consider converting A to a dense
+    matrix and using scipy.linalg.solve or its variants.
     """
-    if isspmatrix( b ):
-        b = b.toarray()
-
-    if b.ndim > 1:
-        if max( b.shape ) == b.size:
-            b = b.squeeze()
-        else:
-            raise ValueError, "rhs must be a vector (has shape %s)" % (b.shape,)
-
     if not (isspmatrix_csc(A) or isspmatrix_csr(A)):
         A = csc_matrix(A)
-        warn('spsolve requires CSC or CSR matrix format', SparseEfficiencyWarning)
+        warn('spsolve requires A be CSC or CSR matrix format',
+                SparseEfficiencyWarning)
+
+    # b is a vector only if b have shape (n,) or (n, 1)
+    b_is_sparse = isspmatrix(b)
+    if not b_is_sparse:
+        b = asarray(b)
+    b_is_vector = ((b.ndim == 1) or (b.ndim == 2 and b.shape[1] == 1))
 
     A.sort_indices()
-    A = A.asfptype()  #upcast to a floating point format
+    A = A.asfptype()  # upcast to a floating point format
 
+    # validate input shapes
     M, N = A.shape
     if (M != N):
-        raise ValueError, "matrix must be square (has shape %s)" % (M,N)
-    if M != b.size:
-        raise ValueError, "matrix - rhs size mismatch (%s - %s)"\
-              % (A.shape, b.size)
+        raise ValueError("matrix must be square (has shape %s)" % ((M, N),))
+
+    if M != b.shape[0]:
+        raise ValueError("matrix - rhs dimension mismatch (%s - %s)"
+                         % (A.shape, b.shape[0]))
 
     use_umfpack = use_umfpack and useUmfpack
 
-    if isUmfpack and use_umfpack:
-        if noScikit:
-            warn( 'scipy.sparse.linalg.dsolve.umfpack will be removed,'\
-                    ' install scikits.umfpack instead', DeprecationWarning )
-        if A.dtype.char not in 'dD':
-            raise ValueError, "convert matrix data to double, please, using"\
-                  " .astype(), or set linsolve.useUmfpack = False"
-
-        b = asarray(b, dtype=A.dtype).reshape(-1)
-
-        family = {'d' : 'di', 'D' : 'zi'}
-        umf = umfpack.UmfpackContext( family[A.dtype.char] )
-        return umf.linsolve( umfpack.UMFPACK_A, A, b,
-                             autoTranspose = True )
-
-    else:
-        if isspmatrix_csc(A):
-            flag = 1 # CSC format
-        elif isspmatrix_csr(A):
-            flag = 0 # CSR format
+    if b_is_vector and use_umfpack:
+        if b_is_sparse:
+            b_vec = b.toarray()
         else:
-            A = csc_matrix(A)
-            flag = 1
+            b_vec = b
+        b_vec = asarray(b_vec, dtype=A.dtype).ravel()
+
+        if noScikit:
+            raise RuntimeError('Scikits.umfpack not installed.')
+
+        if A.dtype.char not in 'dD':
+            raise ValueError("convert matrix data to double, please, using"
+                  " .astype(), or set linsolve.useUmfpack = False")
+
+        family = {'d': 'di', 'D': 'zi'}
+        umf = umfpack.UmfpackContext(family[A.dtype.char])
+        x = umf.linsolve(umfpack.UMFPACK_A, A, b_vec,
+                         autoTranspose=True)
+    else:
+        if b_is_vector and b_is_sparse:
+            b = b.toarray()
+            b_is_sparse = False
+
+        if not b_is_sparse:
+            if isspmatrix_csc(A):
+                flag = 1  # CSC format
+            else:
+                flag = 0  # CSR format
+
+            options = dict(ColPerm=permc_spec)
+            x, info = _superlu.gssv(N, A.nnz, A.data, A.indices, A.indptr,
+                                    b, flag, options=options)
+            if info != 0:
+                warn("Matrix is exactly singular", MatrixRankWarning)
+                x.fill(np.nan)
+            if b_is_vector:
+                x = x.ravel()
+        else:
+            # b is sparse
+            Afactsolve = factorized(A)
+            tempj = empty(M, dtype=int)
+            x = A.__class__(b.shape)
+
+            if not (isspmatrix_csr(b) or isspmatrix_csc(b)):
+                warn('spsolve requires sparse b be in CSC or CSR matrix format',
+                     SparseEfficiencyWarning)
+                b = csc_matrix(b)
+
+            for j in range(b.shape[1]):
+                col = b[:, j].toarray()
+                xj = Afactsolve(ravel(col))
+                w = nonzero(xj)[0]
+                tempj.fill(j)
+                x = x + A.__class__((xj[w], (w, tempj[:len(w)])),
+                                    shape=b.shape, dtype=A.dtype)
+
+    return x
 
-        b = asarray(b, dtype=A.dtype)
-        options = dict(ColPerm=permc_spec)
-        return _superlu.gssv(N, A.nnz, A.data, A.indices, A.indptr, b, flag,
-                             options=options)[0]
 
 def splu(A, permc_spec=None, diag_pivot_thresh=None,
          drop_tol=None, relax=None, panel_size=None, options=dict()):
@@ -105,9 +175,8 @@ def splu(A, permc_spec=None, diag_pivot_thresh=None,
 
     Parameters
     ----------
-    A
+    A : sparse matrix
         Sparse matrix to factorize. Should be in CSR or CSC format.
-
     permc_spec : str, optional
         How to permute the columns of the matrix for sparsity preservation.
         (default: 'COLAMD')
@@ -137,7 +206,7 @@ def splu(A, permc_spec=None, diag_pivot_thresh=None,
 
     Returns
     -------
-    invA : scipy.sparse.linalg.dsolve._superlu.SciPyLUType
+    invA : scipy.sparse.linalg.SuperLU
         Object, which has a ``solve`` method.
 
     See also
@@ -159,11 +228,11 @@ def splu(A, permc_spec=None, diag_pivot_thresh=None,
         warn('splu requires CSC matrix format', SparseEfficiencyWarning)
 
     A.sort_indices()
-    A = A.asfptype()  #upcast to a floating point format
+    A = A.asfptype()  # upcast to a floating point format
 
     M, N = A.shape
     if (M != N):
-        raise ValueError, "can only factor square matrices" #is this true?
+        raise ValueError("can only factor square matrices")  # is this true?
 
     _options = dict(DiagPivotThresh=diag_pivot_thresh, ColPerm=permc_spec,
                     PanelSize=panel_size, Relax=relax)
@@ -172,18 +241,18 @@ def splu(A, permc_spec=None, diag_pivot_thresh=None,
     return _superlu.gstrf(N, A.nnz, A.data, A.indices, A.indptr,
                           ilu=False, options=_options)
 
+
 def spilu(A, drop_tol=None, fill_factor=None, drop_rule=None, permc_spec=None,
           diag_pivot_thresh=None, relax=None, panel_size=None, options=None):
     """
-    Compute an incomplete LU decomposition for a sparse, square matrix A.
+    Compute an incomplete LU decomposition for a sparse, square matrix.
 
-    The resulting object is an approximation to the inverse of A.
+    The resulting object is an approximation to the inverse of `A`.
 
     Parameters
     ----------
-    A
+    A : (N, N) array_like
         Sparse matrix to factorize
-
     drop_tol : float, optional
         Drop tolerance (0 <= tol <= 1) for an incomplete LU decomposition.
         (default: 1e-4)
@@ -204,7 +273,7 @@ def spilu(A, drop_tol=None, fill_factor=None, drop_rule=None, permc_spec=None,
 
     Returns
     -------
-    invA_approx : scipy.sparse.linalg.dsolve._superlu.SciPyLUType
+    invA_approx : scipy.sparse.linalg.SuperLU
         Object, which has a ``solve`` method.
 
     See also
@@ -214,26 +283,21 @@ def spilu(A, drop_tol=None, fill_factor=None, drop_rule=None, permc_spec=None,
     Notes
     -----
     To improve the better approximation to the inverse, you may need to
-    increase ``fill_factor`` AND decrease ``drop_tol``.
+    increase `fill_factor` AND decrease `drop_tol`.
 
     This function uses the SuperLU library.
 
-    References
-    ----------
-    .. [SLU] SuperLU http://crd.lbl.gov/~xiaoye/SuperLU/
-
     """
-
     if not isspmatrix_csc(A):
         A = csc_matrix(A)
         warn('splu requires CSC matrix format', SparseEfficiencyWarning)
 
     A.sort_indices()
-    A = A.asfptype()  #upcast to a floating point format
+    A = A.asfptype()  # upcast to a floating point format
 
     M, N = A.shape
     if (M != N):
-        raise ValueError, "can only factor square matrices" #is this true?
+        raise ValueError("can only factor square matrices")  # is this true?
 
     _options = dict(ILU_DropRule=drop_rule, ILU_DropTol=drop_tol,
                     ILU_FillFactor=fill_factor,
@@ -244,40 +308,59 @@ def spilu(A, drop_tol=None, fill_factor=None, drop_rule=None, permc_spec=None,
     return _superlu.gstrf(N, A.nnz, A.data, A.indices, A.indptr,
                           ilu=True, options=_options)
 
-def factorized( A ):
+
+def factorized(A):
     """
     Return a fuction for solving a sparse linear system, with A pre-factorized.
 
-    Example:
-      solve = factorized( A ) # Makes LU decomposition.
-      x1 = solve( rhs1 ) # Uses the LU factors.
-      x2 = solve( rhs2 ) # Uses again the LU factors.
+    Parameters
+    ----------
+    A : (N, N) array_like
+        Input.
+
+    Returns
+    -------
+    solve : callable
+        To solve the linear system of equations given in `A`, the `solve`
+        callable should be passed an ndarray of shape (N,).
+
+    Examples
+    --------
+    >>> A = np.array([[ 3. ,  2. , -1. ],
+                      [ 2. , -2. ,  4. ],
+                      [-1. ,  0.5, -1. ]])
+
+    >>> solve = factorized( A ) # Makes LU decomposition.
+
+    >>> rhs1 = np.array([1,-2,0])
+    >>> x1 = solve( rhs1 ) # Uses the LU factors.
+    array([ 1., -2., -2.])
+
     """
-    if isUmfpack and useUmfpack:
+    if useUmfpack:
         if noScikit:
-            warn( 'scipy.sparse.linalg.dsolve.umfpack will be removed,'\
-                    ' install scikits.umfpack instead', DeprecationWarning )
+            raise RuntimeError('Scikits.umfpack not installed.')
 
         if not isspmatrix_csc(A):
             A = csc_matrix(A)
             warn('splu requires CSC matrix format', SparseEfficiencyWarning)
 
         A.sort_indices()
-        A = A.asfptype()  #upcast to a floating point format
+        A = A.asfptype()  # upcast to a floating point format
 
         if A.dtype.char not in 'dD':
-            raise ValueError, "convert matrix data to double, please, using"\
-                  " .astype(), or set linsolve.useUmfpack = False"
+            raise ValueError("convert matrix data to double, please, using"
+                  " .astype(), or set linsolve.useUmfpack = False")
 
-        family = {'d' : 'di', 'D' : 'zi'}
-        umf = umfpack.UmfpackContext( family[A.dtype.char] )
+        family = {'d': 'di', 'D': 'zi'}
+        umf = umfpack.UmfpackContext(family[A.dtype.char])
 
         # Make LU decomposition.
-        umf.numeric( A )
+        umf.numeric(A)
 
-        def solve( b ):
-            return umf.solve( umfpack.UMFPACK_A, A, b, autoTranspose = True )
+        def solve(b):
+            return umf.solve(umfpack.UMFPACK_A, A, b, autoTranspose=True)
 
         return solve
     else:
-        return splu( A ).solve
+        return splu(A).solve
