@@ -1,13 +1,15 @@
 """Iterative methods for solving linear systems"""
 
+from __future__ import division, print_function, absolute_import
+
 __all__ = ['bicg','bicgstab','cg','cgs','gmres','qmr']
 
-import _iterative
+from . import _iterative
 import numpy as np
 
 from scipy.sparse.linalg.interface import LinearOperator
 from scipy.lib.decorator import decorator
-from utils import make_system
+from .utils import make_system
 
 _type_conv = {'f':'s', 'd':'d', 'F':'c', 'D':'z'}
 
@@ -59,7 +61,7 @@ xtype : {'f','d','F','D'}
     will compute A.matvec(x0) to get a typecode.   To save the extra
     computation when A does not have a typecode attribute use xtype=0
     for the same type as b or use xtype='f','d','F',or 'D'.
-    This parameter has been superceeded by LinearOperator.
+    This parameter has been superseded by LinearOperator.
 
 """
 
@@ -72,6 +74,7 @@ def set_docstring(header, Ainfo, footer=''):
         return fn
     return combine
 
+
 @decorator
 def non_reentrant(func, *a, **kw):
     d = func.__dict__
@@ -83,6 +86,7 @@ def non_reentrant(func, *a, **kw):
     finally:
         d['__entered'] = False
 
+
 @set_docstring('Use BIConjugate Gradient iteration to solve A x = b',
                'The real or complex N-by-N matrix of the linear system\n'
                'It is required that the linear operator can produce\n'
@@ -98,7 +102,7 @@ def bicg(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=Non
     matvec, rmatvec = A.matvec, A.rmatvec
     psolve, rpsolve = M.matvec, M.rmatvec
     ltr = _type_conv[x.dtype.char]
-    revcom   = getattr(_iterative, ltr + 'bicgrevcom')
+    revcom = getattr(_iterative, ltr + 'bicgrevcom')
     stoptest = getattr(_iterative, ltr + 'stoptest2')
 
     resid = tol
@@ -143,11 +147,12 @@ def bicg(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=Non
         ijob = 2
 
     if info > 0 and iter_ == maxiter and resid > tol:
-        #info isn't set appropriately otherwise
+        # info isn't set appropriately otherwise
         info = iter_
 
     return postprocess(x), info
 
+
 @set_docstring('Use BIConjugate Gradient STABilized iteration to solve A x = b',
                'The real or complex N-by-N matrix of the linear system\n'
                '``A`` must represent a hermitian, positive definite matrix')
@@ -162,7 +167,7 @@ def bicgstab(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback
     matvec = A.matvec
     psolve = M.matvec
     ltr = _type_conv[x.dtype.char]
-    revcom   = getattr(_iterative, ltr + 'bicgstabrevcom')
+    revcom = getattr(_iterative, ltr + 'bicgstabrevcom')
     stoptest = getattr(_iterative, ltr + 'stoptest2')
 
     resid = tol
@@ -190,8 +195,6 @@ def bicgstab(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback
             work[slice2] *= sclr2
             work[slice2] += sclr1*matvec(work[slice1])
         elif (ijob == 2):
-            if psolve is None:
-                psolve = get_psolve(A)
             work[slice1] = psolve(work[slice2])
         elif (ijob == 3):
             work[slice2] *= sclr2
@@ -204,11 +207,12 @@ def bicgstab(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback
         ijob = 2
 
     if info > 0 and iter_ == maxiter and resid > tol:
-        #info isn't set appropriately otherwise
+        # info isn't set appropriately otherwise
         info = iter_
 
     return postprocess(x), info
 
+
 @set_docstring('Use Conjugate Gradient iteration to solve A x = b',
                'The real or complex N-by-N matrix of the linear system\n'
                '``A`` must represent a hermitian, positive definite matrix')
@@ -223,7 +227,7 @@ def cg(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=None)
     matvec = A.matvec
     psolve = M.matvec
     ltr = _type_conv[x.dtype.char]
-    revcom   = getattr(_iterative, ltr + 'cgrevcom')
+    revcom = getattr(_iterative, ltr + 'cgrevcom')
     stoptest = getattr(_iterative, ltr + 'stoptest2')
 
     resid = tol
@@ -262,9 +266,8 @@ def cg(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=None)
             bnrm2, resid, info = stoptest(work[slice1], b, bnrm2, tol, info)
         ijob = 2
 
-
     if info > 0 and iter_ == maxiter and resid > tol:
-        #info isn't set appropriately otherwise
+        # info isn't set appropriately otherwise
         info = iter_
 
     return postprocess(x), info
@@ -283,7 +286,7 @@ def cgs(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=None
     matvec = A.matvec
     psolve = M.matvec
     ltr = _type_conv[x.dtype.char]
-    revcom   = getattr(_iterative, ltr + 'cgsrevcom')
+    revcom = getattr(_iterative, ltr + 'cgsrevcom')
     stoptest = getattr(_iterative, ltr + 'stoptest2')
 
     resid = tol
@@ -323,11 +326,12 @@ def cgs(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=None
         ijob = 2
 
     if info > 0 and iter_ == maxiter and resid > tol:
-        #info isn't set appropriately otherwise
+        # info isn't set appropriately otherwise
         info = iter_
 
     return postprocess(x), info
 
+
 @non_reentrant
 def gmres(A, b, x0=None, tol=1e-5, restart=None, maxiter=None, xtype=None, M=None, callback=None, restrt=None):
     """
@@ -362,8 +366,9 @@ def gmres(A, b, x0=None, tol=1e-5, restart=None, maxiter=None, xtype=None, M=Non
         iteration cost, but may be necessary for convergence.
         Default is 20.
     maxiter : int, optional
-        Maximum number of iterations.  Iteration will stop after maxiter
-        steps even if the specified tolerance has not been achieved.
+        Maximum number of iterations (restart cycles).  Iteration will stop
+        after maxiter steps even if the specified tolerance has not been
+        achieved.
     xtype : {'f','d','F','D'}
         This parameter is DEPRECATED --- avoid using it.
 
@@ -372,7 +377,7 @@ def gmres(A, b, x0=None, tol=1e-5, restart=None, maxiter=None, xtype=None, M=Non
         will compute A.matvec(x0) to get a typecode.   To save the extra
         computation when A does not have a typecode attribute use xtype=0
         for the same type as b or use xtype='f','d','F',or 'D'.
-        This parameter has been superceeded by LinearOperator.
+        This parameter has been superseded by LinearOperator.
     M : {sparse matrix, dense matrix, LinearOperator}
         Inverse of the preconditioner of A.  M should approximate the
         inverse of A and be easy to solve for (see Notes).  Effective
@@ -391,10 +396,10 @@ def gmres(A, b, x0=None, tol=1e-5, restart=None, maxiter=None, xtype=None, M=Non
 
     Notes
     -----
-    A preconditioner, P, is chosen such that P is close to A but easy to solve for.
-    The preconditioner parameter required by this routine is ``M = P^-1``.
-    The inverse should preferably not be calculated explicitly.  Rather, use the
-    following template to produce M::
+    A preconditioner, P, is chosen such that P is close to A but easy to solve
+    for. The preconditioner parameter required by this routine is
+    ``M = P^-1``. The inverse should preferably not be calculated
+    explicitly.  Rather, use the following template to produce M::
 
       # Construct a linear operator that computes P^-1 * x.
       import scipy.sparse.linalg as spla
@@ -423,13 +428,13 @@ def gmres(A, b, x0=None, tol=1e-5, restart=None, maxiter=None, xtype=None, M=Non
     matvec = A.matvec
     psolve = M.matvec
     ltr = _type_conv[x.dtype.char]
-    revcom   = getattr(_iterative, ltr + 'gmresrevcom')
+    revcom = getattr(_iterative, ltr + 'gmresrevcom')
     stoptest = getattr(_iterative, ltr + 'stoptest2')
 
     resid = tol
     ndx1 = 1
     ndx2 = -1
-    work  = np.zeros((6+restrt)*n,dtype=x.dtype)
+    work = np.zeros((6+restrt)*n,dtype=x.dtype)
     work2 = np.zeros((restrt+1)*(2*restrt+2),dtype=x.dtype)
     ijob = 1
     info = 0
@@ -444,11 +449,11 @@ def gmres(A, b, x0=None, tol=1e-5, restart=None, maxiter=None, xtype=None, M=Non
         olditer = iter_
         x, iter_, resid, info, ndx1, ndx2, sclr1, sclr2, ijob = \
            revcom(b, x, restrt, work, work2, iter_, resid, info, ndx1, ndx2, ijob)
-        #if callback is not None and iter_ > olditer:
+        # if callback is not None and iter_ > olditer:
         #    callback(x)
         slice1 = slice(ndx1-1, ndx1-1+n)
         slice2 = slice(ndx2-1, ndx2-1+n)
-        if (ijob == -1): # gmres success, update last residual
+        if (ijob == -1):  # gmres success, update last residual
             if resid_ready and callback is not None:
                 callback(resid)
                 resid_ready = False
@@ -459,7 +464,7 @@ def gmres(A, b, x0=None, tol=1e-5, restart=None, maxiter=None, xtype=None, M=Non
             work[slice2] += sclr1*matvec(x)
         elif (ijob == 2):
             work[slice1] = psolve(work[slice2])
-            if not first_pass and old_ijob==3:
+            if not first_pass and old_ijob == 3:
                 resid_ready = True
 
             first_pass = False
@@ -484,7 +489,7 @@ def gmres(A, b, x0=None, tol=1e-5, restart=None, maxiter=None, xtype=None, M=Non
             break
 
     if info >= 0 and resid > tol:
-        #info isn't set appropriately otherwise
+        # info isn't set appropriately otherwise
         info = maxiter
 
     return postprocess(x), info
@@ -540,7 +545,7 @@ def qmr(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M1=None, M2=None, cal
         will compute A.matvec(x0) to get a typecode.   To save the extra
         computation when A does not have a typecode attribute use xtype=0
         for the same type as b or use xtype='f','d','F',or 'D'.
-        This parameter has been superceeded by LinearOperator.
+        This parameter has been superseded by LinearOperator.
 
     See Also
     --------
@@ -554,10 +559,13 @@ def qmr(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M1=None, M2=None, cal
         if hasattr(A_,'psolve'):
             def left_psolve(b):
                 return A_.psolve(b,'left')
+
             def right_psolve(b):
                 return A_.psolve(b,'right')
+
             def left_rpsolve(b):
                 return A_.rpsolve(b,'left')
+
             def right_rpsolve(b):
                 return A_.rpsolve(b,'right')
             M1 = LinearOperator(A.shape, matvec=left_psolve, rmatvec=left_rpsolve)
@@ -573,7 +581,7 @@ def qmr(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M1=None, M2=None, cal
         maxiter = n*10
 
     ltr = _type_conv[x.dtype.char]
-    revcom   = getattr(_iterative, ltr + 'qmrrevcom')
+    revcom = getattr(_iterative, ltr + 'qmrrevcom')
     stoptest = getattr(_iterative, ltr + 'stoptest2')
 
     resid = tol
@@ -622,7 +630,7 @@ def qmr(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M1=None, M2=None, cal
         ijob = 2
 
     if info > 0 and iter_ == maxiter and resid > tol:
-        #info isn't set appropriately otherwise
+        # info isn't set appropriately otherwise
         info = iter_
 
     return postprocess(x), info
