@@ -15,7 +15,7 @@ stored in a rectangular array.
    :toctree: generated/
 
    pdist   -- pairwise distances between observation vectors.
-   cdist   -- distances between between two collections of observation vectors
+   cdist   -- distances between two collections of observation vectors
    squareform -- convert distance matrix to a condensed one and vice versa
 
 Predicates for checking the validity of distance matrices, both
@@ -68,12 +68,12 @@ from __future__ import division, print_function, absolute_import
 
 import warnings
 import numpy as np
-from numpy.linalg import norm
 
 from scipy.lib.six import callable, string_types
-from scipy.lib.six.moves import xrange
+from scipy.lib.six import xrange
 
 from . import _distance_wrap
+from ..linalg import norm
 import collections
 
 
@@ -249,10 +249,19 @@ def sqeuclidean(u, v):
         The squared Euclidean distance between vectors `u` and `v`.
 
     """
-    u = _validate_vector(u)
-    v = _validate_vector(v)
-    dist = ((u - v) ** 2).sum()
-    return dist
+    # Preserve float dtypes, but convert everything else to np.float64
+    # for stability.
+    utype, vtype = None, None
+    if not (hasattr(u, "dtype") and np.issubdtype(u.dtype, np.inexact)):
+        utype = np.float64
+    if not (hasattr(v, "dtype") and np.issubdtype(v.dtype, np.inexact)):
+        vtype = np.float64
+
+    u = _validate_vector(u, dtype=utype)
+    v = _validate_vector(v, dtype=vtype)
+    u_v = u - v
+
+    return np.dot(u_v, u_v)
 
 
 def cosine(u, v):
@@ -1211,8 +1220,7 @@ def pdist(X, metric='euclidean', p=2, w=None, V=None, VI=None):
         if mstr in set(['euclidean', 'euclid', 'eu', 'e']):
             _distance_wrap.pdist_euclidean_wrap(_convert_to_double(X), dm)
         elif mstr in set(['sqeuclidean', 'sqe', 'sqeuclid']):
-            _distance_wrap.pdist_euclidean_wrap(_convert_to_double(X), dm)
-            dm = dm ** 2.0
+            _distance_wrap.pdist_sqeuclidean_wrap(_convert_to_double(X), dm)
         elif mstr in set(['cityblock', 'cblock', 'cb', 'c']):
             _distance_wrap.pdist_city_block_wrap(X, dm)
         elif mstr in set(['hamming', 'hamm', 'ha', 'h']):
@@ -1873,7 +1881,7 @@ def cdist(XA, XB, metric='euclidean', p=2, V=None, VI=None, w=None):
     22. ``Y = cdist(XA, XB, 'wminkowski')``
 
        Computes the weighted Minkowski distance between the
-       vectors. (see sokalsneath function documentation)
+       vectors. (see wminkowski function documentation)
 
     23. ``Y = cdist(XA, XB, f)``
 
@@ -2001,9 +2009,8 @@ def cdist(XA, XB, metric='euclidean', p=2, V=None, VI=None, w=None):
             _distance_wrap.cdist_euclidean_wrap(_convert_to_double(XA),
                                                 _convert_to_double(XB), dm)
         elif mstr in set(['sqeuclidean', 'sqe', 'sqeuclid']):
-            _distance_wrap.cdist_euclidean_wrap(_convert_to_double(XA),
+            _distance_wrap.cdist_sqeuclidean_wrap(_convert_to_double(XA),
                                                 _convert_to_double(XB), dm)
-            dm **= 2.0
         elif mstr in set(['cityblock', 'cblock', 'cb', 'c']):
             _distance_wrap.cdist_city_block_wrap(_convert_to_double(XA),
                                                  _convert_to_double(XB), dm)
