@@ -238,19 +238,19 @@ class KDTree(object):
     class node(object):
         if sys.version_info[0] >= 3:
             def __lt__(self, other):
-                id(self) < id(other)
+                return id(self) < id(other)
 
             def __gt__(self, other):
-                id(self) > id(other)
+                return id(self) > id(other)
 
             def __le__(self, other):
-                id(self) <= id(other)
+                return id(self) <= id(other)
 
             def __ge__(self, other):
-                id(self) >= id(other)
+                return id(self) >= id(other)
 
             def __eq__(self, other):
-                id(self) == id(other)
+                return id(self) == id(other)
 
     class leafnode(node):
         def __init__(self, idx):
