@@ -1,16 +1,16 @@
 from __future__ import division, print_function, absolute_import
 
 import os
-import sys
+import copy
 
 import numpy as np
 from numpy.testing import assert_equal, assert_almost_equal, run_module_suite,\
      assert_, dec, assert_allclose, assert_array_equal, assert_raises
-from scipy.lib.six.moves import xrange
+from scipy.lib.six import xrange
 
-import copy
 import scipy.spatial.qhull as qhull
 from scipy.spatial import cKDTree as KDTree
+from scipy.lib._version import NumpyVersion
 
 
 def sorted_tuple(x):
@@ -66,7 +66,7 @@ pathological_data_2 = np.array([
 ])
 
 bug_2850_chunks = [np.random.rand(10, 2),
-                   np.array([[0,0], [0,1], [1,0], [1,1]]) # add corners
+                   np.array([[0,0], [0,1], [1,0], [1,1]])  # add corners
                    ]
 
 # same with some additional chunks
@@ -92,6 +92,7 @@ INCREMENTAL_DATASETS = {
     'bug-2850-2': (bug_2850_chunks_2, None),
 }
 
+
 def _add_inc_data(name, chunksize):
     """
     Generate incremental datasets from basic data sets
@@ -179,7 +180,7 @@ class TestUtilities(object):
         # |1 \|
         # +---+
 
-        assert_equal(tri.vertices, [[3, 1, 2], [3, 1, 0]])
+        assert_equal(tri.vertices, [[1, 3, 2], [3, 1, 0]])
 
         for p in [(0.25, 0.25, 1),
                   (0.75, 0.75, 0),
@@ -227,7 +228,7 @@ class TestUtilities(object):
         # |1 \|
         # +---+
 
-        assert_equal(tri.convex_hull, [[1, 2], [3, 2], [1, 0], [3, 0]])
+        assert_equal(tri.convex_hull, [[3, 2], [1, 2], [1, 0], [3, 0]])
 
     def _check_barycentric_transforms(self, tri, err_msg="",
                                       unit_cube=False,
@@ -278,7 +279,8 @@ class TestUtilities(object):
             ok = (j != -1) | at_boundary
             assert_(ok.all(), "%s %s" % (err_msg, np.where(~ok)))
 
-    @dec.skipif(np.version.short_version < '1.6', "No einsum in numpy 1.5.x")
+    @dec.skipif(NumpyVersion(np.__version__) < '1.6.0',
+                "No einsum in numpy 1.5.x")
     def test_degenerate_barycentric_transforms(self):
         # The triangulation should not produce invalid barycentric
         # transforms that stump the simplex finding
@@ -297,7 +299,8 @@ class TestUtilities(object):
         self._check_barycentric_transforms(tri)
 
     @dec.slow
-    @dec.skipif(np.version.short_version < '1.6', "No einsum in numpy 1.5.x")
+    @dec.skipif(NumpyVersion(np.__version__) < '1.6.0',
+                "No einsum in numpy 1.5.x")
     def test_more_barycentric_transforms(self):
         # Triangulate some "nasty" grids
 
@@ -393,6 +396,9 @@ class TestDelaunay(object):
     Check that triangulation works.
 
     """
+    def test_masked_array_fails(self):
+        masked_array = np.ma.masked_all(1)
+        assert_raises(ValueError, qhull.Delaunay, masked_array)
 
     def test_nd_simplex(self):
         # simple smoke test: triangulate a n-dimensional simplex
@@ -414,7 +420,7 @@ class TestDelaunay(object):
         points = np.array([(0,0), (0,1), (1,1), (1,0)], dtype=np.double)
         tri = qhull.Delaunay(points)
 
-        assert_equal(tri.vertices, [[3, 1, 2], [3, 1, 0]])
+        assert_equal(tri.vertices, [[1, 3, 2], [3, 1, 0]])
         assert_equal(tri.neighbors, [[-1, -1, 1], [-1, -1, 0]])
 
     def test_duplicate_points(self):
@@ -469,7 +475,7 @@ class TestDelaunay(object):
         points = [(0, 0), (0, 1), (1, 0), (0.5, 0.5), (1.1, 1.1)]
         tri = qhull.Delaunay(points, furthest_site=True)
 
-        expected = np.array([(1, 4, 0), (2, 4, 0)])  # from Qhull
+        expected = np.array([(1, 4, 0), (4, 2, 0)])  # from Qhull
         assert_array_equal(tri.simplices, expected)
 
     def test_incremental(self):
@@ -568,6 +574,10 @@ def assert_hulls_equal(points, facets_1, facets_2):
 
 
 class TestConvexHull:
+    def test_masked_array_fails(self):
+        masked_array = np.ma.masked_all(1)
+        assert_raises(ValueError, qhull.ConvexHull, masked_array)
+
     def test_hull_consistency_tri(self):
         # Check that a convex hull returned by qhull in ndim
         # and the hull constructed from ndim delaunay agree
@@ -627,6 +637,10 @@ class TestConvexHull:
 
 
 class TestVoronoi:
+    def test_masked_array_fails(self):
+        masked_array = np.ma.masked_all(1)
+        assert_raises(ValueError, qhull.Voronoi, masked_array)
+
     def test_simple(self):
         # Simple case with known Voronoi diagram
         points = [(0, 0), (0, 1), (0, 2),
