@@ -2,11 +2,11 @@ from __future__ import division, print_function, absolute_import
 
 from numpy.testing import assert_array_almost_equal, assert_almost_equal, \
         rand, TestCase
-from scipy.lib.six.moves import xrange
+from scipy.lib.six import xrange
 import numpy as np
 from numpy import array, sqrt
 import scipy.special.orthogonal as orth
-from scipy.special import gamma
+from scipy.special import gamma, eval_hermite
 
 
 class TestCheby(TestCase):
@@ -134,6 +134,20 @@ class TestHermite(TestCase):
         assert_array_almost_equal(H4.c,he4.c,13)
         assert_array_almost_equal(H5.c,he5.c,13)
 
+    def test_h_roots(self):
+        # this test is copied from numpy's TestGauss in test_hermite.py
+        x, w = orth.h_roots(100)
+
+        n = np.arange(100)
+        v = eval_hermite(n[:, np.newaxis], x[np.newaxis,:])
+        vv = np.dot(v*w, v.T)
+        vd = 1 / np.sqrt(vv.diagonal())
+        vv = vd[:, np.newaxis] * vv * vd
+        assert_almost_equal(vv, np.eye(100))
+
+        # check that the integral of 1 is correct
+        assert_almost_equal(w.sum(), np.sqrt(np.pi))
+
 
 class _test_sh_legendre(TestCase):
 
