@@ -1,20 +1,21 @@
 from __future__ import division, print_function, absolute_import
 
-from distutils.version import LooseVersion
 import sys
 
 import numpy as np
 from numpy.testing import assert_, assert_allclose, dec
 import scipy.special.orthogonal as orth
 
+from scipy.lib._version import NumpyVersion
 from scipy.special._testutils import FuncData
 
+
 # Early Numpy versions have bugs in ufunc keyword argument parsing
 numpy_version_requirement = dec.skipif(
-    LooseVersion(np.version.version) < LooseVersion('1.6')
-    and sys.version_info[0] >= 3,
+    NumpyVersion(np.__version__) < '1.6.0' and sys.version_info[0] >= 3,
     "Bug in Numpy < 1.6 on Python 3")
 
+
 def test_eval_chebyt():
     n = np.arange(0, 10000, 7)
     x = 2*np.random.rand() - 1
@@ -23,6 +24,12 @@ def test_eval_chebyt():
     assert_(np.allclose(v1, v2, rtol=1e-15))
 
 
+def test_eval_genlaguerre_restriction():
+    # check it returns nan for alpha <= -1
+    assert_(np.isnan(orth.eval_genlaguerre(0, -1, 0)))
+    assert_(np.isnan(orth.eval_genlaguerre(0.1, -1, 0)))
+
+
 def test_warnings():
     # ticket 1334
     olderr = np.seterr(all='raise')
@@ -256,3 +263,9 @@ class TestRecurrence(object):
     def test_laguerre(self):
         self.check_poly(orth.eval_laguerre,
                    param_ranges=[], x_range=[0, 100])
+
+    @numpy_version_requirement
+    def test_hermite(self):
+        v = orth.eval_hermite(70, 1.0)
+        a = -1.457076485701412e60
+        assert_allclose(v,a)
