@@ -8,20 +8,23 @@ accelerate_tools contains the interface for on-the-fly building of
 C++ equivalents to Python functions.
 """
 #**************************************************************************#
+from __future__ import absolute_import, print_function
 
 from types import InstanceType, XRangeType
 import inspect
-import scipy.weave.md5_load as md5
+from hashlib import sha256
 import scipy.weave as weave
+from numpy.testing import assert_
+
+from .bytecodecompiler import CXXCoder,Type_Descriptor,Function_Descriptor
 
-from bytecodecompiler import CXXCoder,Type_Descriptor,Function_Descriptor
 
 def CStr(s):
     "Hacky way to get legal C string from Python string"
     if s is None:
         return '""'
-    assert isinstance(s, str), "only None and string allowed"
-    r = repr('"'+s) # Better for embedded quotes
+    assert_(isinstance(s, str), msg="only None and string allowed")
+    r = repr('"'+s)  # Better for embedded quotes
     return '"'+r[2:-1]+'"'
 
 
@@ -32,10 +35,10 @@ class Instance(Type_Descriptor):
     cxxtype = 'PyObject*'
 
     def __init__(self,prototype):
-        self.prototype  = prototype
+        self.prototype = prototype
 
     def check(self,s):
-        return "PyInstance_Check(%s)"%s
+        return "PyInstance_Check(%s)" % s
 
     def inbound(self,s):
         return s
@@ -46,40 +49,48 @@ class Instance(Type_Descriptor):
     def get_attribute(self,name):
         proto = getattr(self.prototype,name)
         T = lookup_type(proto)
-        code = 'tempPY = PyObject_GetAttrString(%%(rhs)s,"%s");\n'%name
+        code = 'tempPY = PyObject_GetAttrString(%%(rhs)s,"%s");\n' % name
         convert = T.inbound('tempPY')
-        code += '%%(lhsType)s %%(lhs)s = %s;\n'%convert
+        code += '%%(lhsType)s %%(lhs)s = %s;\n' % convert
         return T,code
 
     def set_attribute(self,name):
         proto = getattr(self.prototype,name)
         T = lookup_type(proto)
         convert,owned = T.outbound('%(rhs)s')
-        code = 'tempPY = %s;'%convert
+        code = 'tempPY = %s;' % convert
         if not owned:
             code += ' Py_INCREF(tempPY);'
-        code += ' PyObject_SetAttrString(%%(lhs)s,"%s",tempPY);'%name
+        code += ' PyObject_SetAttrString(%%(lhs)s,"%s",tempPY);' % name
         code += ' Py_DECREF(tempPY);\n'
         return T,code
 
 ##################################################################
 #                          CLASS BASIC                           #
 ##################################################################
+
+
 class Basic(Type_Descriptor):
     owned = 1
+
     def check(self,s):
-        return "%s(%s)"%(self.checker,s)
+        return "%s(%s)" % (self.checker,s)
+
     def inbound(self,s):
-        return "%s(%s)"%(self.inbounder,s)
+        return "%s(%s)" % (self.inbounder,s)
+
     def outbound(self,s):
-        return "%s(%s)"%(self.outbounder,s),self.owned
+        return "%s(%s)" % (self.outbounder,s),self.owned
+
 
 class Basic_Number(Basic):
     def literalizer(self,s):
         return str(s)
+
     def binop(self,symbol,a,b):
-        assert symbol in ['+','-','*','/'],symbol
-        return '%s %s %s'%(a,symbol,b),self
+        assert_(symbol in ['+','-','*','/'], msg=symbol)
+        return '%s %s %s' % (a,symbol,b),self
+
 
 class Integer(Basic_Number):
     cxxtype = "long"
@@ -87,12 +98,14 @@ class Integer(Basic_Number):
     inbounder = "PyInt_AsLong"
     outbounder = "PyInt_FromLong"
 
+
 class Double(Basic_Number):
     cxxtype = "double"
     checker = "PyFloat_Check"
     inbounder = "PyFloat_AsDouble"
     outbounder = "PyFloat_FromDouble"
 
+
 class String(Basic):
     cxxtype = "char*"
     checker = "PyString_Check"
@@ -111,6 +124,7 @@ String = String()
 
 import numpy as np
 
+
 class Vector(Type_Descriptor):
     cxxtype = 'PyArrayObject*'
     refcount = 1
@@ -118,59 +132,69 @@ class Vector(Type_Descriptor):
     module_init_code = 'import_array();\n'
     inbounder = "(PyArrayObject*)"
     outbounder = "(PyObject*)"
-    owned = 0 # Convertion is by casting!
+    owned = 0  # Conversion is by casting!
 
-    prerequisites = Type_Descriptor.prerequisites+\
+    prerequisites = Type_Descriptor.prerequisites + \
                     ['#include "numpy/arrayobject.h"']
     dims = 1
+
     def check(self,s):
-        return "PyArray_Check(%s) && ((PyArrayObject*)%s)->nd == %d &&  ((PyArrayObject*)%s)->descr->type_num == %s"%(
+        return "PyArray_Check(%s) && ((PyArrayObject*)%s)->nd == %d &&  ((PyArrayObject*)%s)->descr->type_num == %s" % (
             s,s,self.dims,s,self.typecode)
 
     def inbound(self,s):
-        return "%s(%s)"%(self.inbounder,s)
+        return "%s(%s)" % (self.inbounder,s)
+
     def outbound(self,s):
-        return "%s(%s)"%(self.outbounder,s),self.owned
+        return "%s(%s)" % (self.outbounder,s),self.owned
 
     def getitem(self,A,v,t):
-        assert self.dims == len(v),'Expect dimension %d'%self.dims
-        code = '*((%s*)(%s->data'%(self.cxxbase,A)
+        assert_(self.dims == len(v), msg='Expect dimension %d' % self.dims)
+        code = '*((%s*)(%s->data' % (self.cxxbase,A)
         for i in range(self.dims):
             # assert that ''t[i]'' is an integer
-            code += '+%s*%s->strides[%d]'%(v[i],A,i)
+            code += '+%s*%s->strides[%d]' % (v[i],A,i)
         code += '))'
         return code,self.pybase
+
     def setitem(self,A,v,t):
         return self.getitem(A,v,t)
 
+
 class matrix(Vector):
     dims = 2
 
+
 class IntegerVector(Vector):
     typecode = 'PyArray_INT'
     cxxbase = 'int'
     pybase = Integer
 
+
 class Integermatrix(matrix):
     typecode = 'PyArray_INT'
     cxxbase = 'int'
     pybase = Integer
 
+
 class LongVector(Vector):
     typecode = 'PyArray_LONG'
     cxxbase = 'long'
     pybase = Integer
 
+
 class Longmatrix(matrix):
     typecode = 'PyArray_LONG'
     cxxbase = 'long'
     pybase = Integer
 
+
 class DoubleVector(Vector):
     typecode = 'PyArray_DOUBLE'
     cxxbase = 'double'
     pybase = Double
 
+
 class Doublematrix(matrix):
     typecode = 'PyArray_DOUBLE'
     cxxbase = 'double'
@@ -211,8 +235,8 @@ XRange = XRange()
 
 
 typedefs = {
-    int : Integer,
-    float : Double,
+    int: Integer,
+    float: Double,
     str: String,
     (np.ndarray,1,int): IntegerVector,
     (np.ndarray,2,int): Integermatrix,
@@ -220,7 +244,7 @@ typedefs = {
     (np.ndarray,2,np.long): Longmatrix,
     (np.ndarray,1,float): DoubleVector,
     (np.ndarray,2,float): Doublematrix,
-    XRangeType : XRange,
+    XRangeType: XRange,
     }
 
 import math
@@ -251,7 +275,6 @@ functiondefs = {
     }
 
 
-
 ##################################################################
 #                      FUNCTION LOOKUP_TYPE                      #
 ##################################################################
@@ -270,10 +293,12 @@ def lookup_type(x):
 ##################################################################
 #                        class ACCELERATE                        #
 ##################################################################
+
+
 class accelerate(object):
 
     def __init__(self, function, *args, **kw):
-        assert inspect.isfunction(function)
+        assert_(inspect.isfunction(function))
         self.function = function
         self.module = inspect.getmodule(function)
         if self.module is None:
@@ -289,7 +314,7 @@ class accelerate(object):
             return self.__cache(*args)
         except TypeError:
             # Figure out type info -- Do as tuple so its hashable
-            signature = tuple( map(lookup_type,args) )
+            signature = tuple(map(lookup_type,args))
 
             # If we know the function, call it
             try:
@@ -302,10 +327,9 @@ class accelerate(object):
 
     def signature(self,*args):
         # Figure out type info -- Do as tuple so its hashable
-        signature = tuple( map(lookup_type,args) )
+        signature = tuple(map(lookup_type,args))
         return self.singleton(signature)
 
-
     def singleton(self,signature):
         identifier = self.identifier(signature)
 
@@ -314,9 +338,9 @@ class accelerate(object):
 
         # See if we have an accelerated version of module
         try:
-            print 'lookup',self.module.__name__+'_weave'
+            print('lookup',self.module.__name__+'_weave')
             accelerated_module = __import__(self.module.__name__+'_weave')
-            print 'have accelerated',self.module.__name__+'_weave'
+            print('have accelerated',self.module.__name__+'_weave')
             fast = getattr(accelerated_module,identifier)
             return fast
         except ImportError:
@@ -340,15 +364,15 @@ class accelerate(object):
         return fast
 
     def identifier(self,signature):
-        # Build an MD5 checksum
+        # Build a (truncated, see gh-3216) SHA-256 checksum
         f = self.function
         co = f.func_code
-        identifier = str(signature)+\
-                     str(co.co_argcount)+\
-                     str(co.co_consts)+\
-                     str(co.co_varnames)+\
+        identifier = str(signature) + \
+                     str(co.co_argcount) + \
+                     str(co.co_consts) + \
+                     str(co.co_varnames) + \
                      co.co_code
-        return 'F'+md5.md5(identifier).hexdigest()
+        return 'F' + sha256(identifier).hexdigest()[:32]
 
     def accelerate(self,signature,identifier):
         P = Python2CXX(self.function,signature,name=identifier)
@@ -360,7 +384,7 @@ class accelerate(object):
                         (self.function.__name__,
                          self.function.func_code.co_argcount,
                          len(args)))
-        signature = tuple( map(lookup_type,args) )
+        signature = tuple(map(lookup_type,args))
         ident = self.function.__name__
         return self.accelerate(signature,ident).function_code()
 
@@ -383,12 +407,12 @@ class Python2CXX(CXXCoder):
 
     def __init__(self,f,signature,name=None):
         # Make sure function is a function
-        assert inspect.isfunction(f)
+        assert_(inspect.isfunction(f))
         # and check the input type signature
-        assert reduce(lambda x,y: x and y,
+        assert_(reduce(lambda x,y: x and y,
                       map(lambda x: isinstance(x,Type_Descriptor),
                           signature),
-                      1),'%s not all type objects'%signature
+                      1), msg='%s not all type objects' % signature)
         self.arg_specs = []
         self.customize = weave.base_info.custom_info()
 
@@ -404,7 +428,7 @@ class Python2CXX(CXXCoder):
         return code
 
     def python_function_definition_code(self):
-        return '{ "%s", wrapper_%s, METH_VARARGS, %s },\n'%(
+        return '{ "%s", wrapper_%s, METH_VARARGS, %s },\n' % (
             self.name,
             self.name,
             CStr(self.function.__doc__))
